# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, TYPE_CHECKING

from azure.core.rest import HttpRequest, HttpResponse
from azure.mgmt.core import ARMPipelineClient

from . import models as _models
from .._serialization import Deserializer, Serializer
from ._configuration import AuthorizationManagementClientConfiguration
from .operations import (
    AlertConfigurationsOperations,
    AlertDefinitionsOperations,
    AlertIncidentsOperations,
    AlertOperationOperations,
    AlertsOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential


class AuthorizationManagementClient:  # pylint: disable=client-accepts-api-version-keyword
    """Role based access control provides you a way to apply granular level policy administration down
    to individual resources or resource groups. These operations enable you to manage role
    assignments. A role assignment grants access to Azure Active Directory users.

    :ivar alerts: AlertsOperations operations
    :vartype alerts: azure.mgmt.authorization.v2022_08_01_preview.operations.AlertsOperations
    :ivar alert_configurations: AlertConfigurationsOperations operations
    :vartype alert_configurations:
     azure.mgmt.authorization.v2022_08_01_preview.operations.AlertConfigurationsOperations
    :ivar alert_definitions: AlertDefinitionsOperations operations
    :vartype alert_definitions:
     azure.mgmt.authorization.v2022_08_01_preview.operations.AlertDefinitionsOperations
    :ivar alert_incidents: AlertIncidentsOperations operations
    :vartype alert_incidents:
     azure.mgmt.authorization.v2022_08_01_preview.operations.AlertIncidentsOperations
    :ivar alert_operation: AlertOperationOperations operations
    :vartype alert_operation:
     azure.mgmt.authorization.v2022_08_01_preview.operations.AlertOperationOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2022-08-01-preview". Note that overriding
     this default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self, credential: "TokenCredential", base_url: str = "https://management.azure.com", **kwargs: Any
    ) -> None:
        self._config = AuthorizationManagementClientConfiguration(credential=credential, **kwargs)
        self._client: ARMPipelineClient = ARMPipelineClient(base_url=base_url, config=self._config, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.alerts = AlertsOperations(
            self._client, self._config, self._serialize, self._deserialize, "2022-08-01-preview"
        )
        self.alert_configurations = AlertConfigurationsOperations(
            self._client, self._config, self._serialize, self._deserialize, "2022-08-01-preview"
        )
        self.alert_definitions = AlertDefinitionsOperations(
            self._client, self._config, self._serialize, self._deserialize, "2022-08-01-preview"
        )
        self.alert_incidents = AlertIncidentsOperations(
            self._client, self._config, self._serialize, self._deserialize, "2022-08-01-preview"
        )
        self.alert_operation = AlertOperationOperations(
            self._client, self._config, self._serialize, self._deserialize, "2022-08-01-preview"
        )

    def _send_request(self, request: HttpRequest, **kwargs: Any) -> HttpResponse:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = client._send_request(request)
        <HttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.HttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, **kwargs)

    def close(self) -> None:
        self._client.close()

    def __enter__(self) -> "AuthorizationManagementClient":
        self._client.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self._client.__exit__(*exc_details)
